function e = compute_error(path_GT,path_result, flag_error_type)
%  Estimating 3D reconstruction errors

% Author: Ravi Garg

% Cite our CVPR 2013 while using the data for 3D reconstruction
% ' Dense Variational Reconstruction of Non-Rigid Surfaces from Monocular Video '
%  Ravi Garg, Anastasios Roussos, Lourdes Agapito
%  IEEE Conference on Computer Vision and Pattern Recognition (CVPR) 2013.


% Please send bug reports and queries to rgarg at eecs dot qmul dot ac dot uk


%% Input: 

% 	path_GT: Path for the Ground Truth data file 

% 	path_result: Path for the file storing the reconstruction result
% 	(Assumes that the 3FxP (or 3FxN) matrix ' S ' in the file stores the
% 	reconstructed shapes.)

% 	flag_error_type: A flag to choose the error measure type. Use:
%       2 -> estimating the correct average RMS and RMS error measures for reconstruction (See NOTE in compute_error_all_methods.m)
%       1 -> replicating Garg etal's error measures.
% 

%% Output:
% 	e:	Stores RMS and Average RMS error measures for the reconstruction.

% Sample usage:
% The following command computes the error scores of Garg etal CVPR 2013 method for sequence 3.
% 	> e = compute_error(' Seq3/GT.mat ' ,  ' Seq3/TB.mat ',  2);


%% load the data
load(path_GT,'S_GT','R_GT','mask');
load(path_result,'S','R');

% estimating number of frames and points
nframe = size(S_GT,1)/3;
npoint = sum(mask(:));

% removing the background points (the grid stucture from mask is used only for applying TV smoothness)
S_GT(:,~mask) = [];
if size(S,2) ~= size(S_GT,2)
    if size(S,2) ~= size(mask(:),1) || size(S,1) ~= size(S_GT,1)
        fprintf('something went wrong');
        keyboard;
    else
        S(:,~mask) = [];
    end
end

% To have equal weight for every frame's error, center of all the ground truth shapes should be zero.
% In the CVPR 2013 version the non zero mean of the ground truth shpes was affecting the mean rms error.
if flag_error_type == 2
    S_GT = S_GT - repmat(mean(S_GT,2),1,npoint);
end
  


%% computing rms error after per frame procrustes based registration
err = zeros(nframe,1);
for i = 1:nframe
     [d,K] = procrustes(S_GT(3*i-2:3*i,:)',S(3*i-2:3*i,:)');
     S(3*i-2:3*i,:) = K';
     err(i) = norm(S(3*i-2:3*i,:) - S_GT(3*i-2:3*i,:) ,'fro')/ norm(S_GT(3*i-2:3*i,:) ,'fro');
end

e.mean_rms = mean(err);
e.rms = rms(err);

end







